#!/bin/bash
#
# programa de instalacin y personalizacin del paquete freevote
#
# <C> Juan Antonio Martnez Junio-2000


[ -x /etc/freevote.conf ] || ( 
	echo "/etc/freevote.conf no existe o no es ejecutable. $0 Abortado"
	exit 1
)

. /etc/freevote.conf

sqlquery () {
	psql -d $DBNAME -F : -h $DBHOST -p $DBPORT -t -A -q -U $DBUSER -c "$*"
}

daily () {
	# Comprueba expiracion de consultas y limpia la base de datos
	# de las encuestas caducadas
	date=`date +"%Y-%m-%d"`
	sqlquery "DELETE FROM consultas WHERE end_convocatoria<'$date';"
}

weekly () {
	# Rotacion de backups y logs
	FILE=${BACKUPDIR}/$DBNAME.sql
	LOG=${LOGDIR}/$DBNAME.log
	[ -f ${FILE}.2 ] && mv $FILE.2 $FILE.3
	[ -f ${FILE}.1 ] && mv $FILE.1 $FILE.2
	[ -f ${FILE}.0 ] && mv $FILE.0 $FILE.1
	[ -f ${FILE} ] && mv $FILE $FILE.0

	[ -f ${LOG}.2 ] && mv $LOG.2 $LOG.3
	[ -f ${LOG}.1 ] && mv $LOG.1 $LOG.2
	[ -f ${LOG}.0 ] && mv $LOG.0 $LOG.1
	[ -f ${LOG} ] && mv $LOG $LOG.0

	# Backup de la database
        echo -e "$DBUSER\n$DBPASSWD\n" | pg_dump -h $DBHOST -p $DBPORT -u $DBNAME | awk 'NR == 1 { next; } NR != 1 { print; }' > $FILE

	# Reindexa y limpia la database
	vacuumdb -d $DBNAME -h $DBHOST -p $DBPORT -U $DBUSER 2> $LOG
}

if [ "Z$1" = "Zdaily" ]; then
	daily
elif [ "Z$1" = "Zweekly" ]; then
	daily
	weekly
else
	echo "Uso: $0 daily | weekly" > /dev/stderr
	exit 1
fi
