#!/bin/bash
#
# programa de instalacin y personalizacin del paquete freevote
#
# <C> Juan Antonio Martnez Junio-2000

cd `dirname $0`
PROGRAM=`basename $0`

CONF=/etc/freevote.conf

if [ $UID -ne 0 ]; then
	echo "Debes ser root para ejecutar este script"
	exit 1
fi

if [ ! -x $CONF ]; then
	echo "$CONF no existe o no es ejecutable. Ejecucin abortada"
	[ -f ${CONF}.dist ] && echo "Edita primero el fichero ${CONF}.dist y guardalo como ${CONF} con permisos 700"
	exit 1
fi

. $CONF

BASE=${FREEVOTE_INST}/lib/freevote

sqlquery () {
        psql -d $DBNAME -F : -h $DBHOST -p $DBPORT -t -A -q -U $DBUSER -c "$*"
} 

if [ "$1" = "--do-all" ]; then
	echo "Free-Vote. Proceso de instalacion"
	$PROGRAM --create-user
	$PROGRAM --restart-db
	$PROGRAM --register-sysop
	if [ "${USE_PGP}" = "PGP" ]; then $PROGRAM --setup-pgp ; fi
	if [ "${USE_PGP}" = "GnuPG" ]; then $PROGRAM --setup-gnupg ; fi
	$PROGRAM --install-files
	echo "Instalacion completa."
	exit 0
fi

if [ "$1" = "--install-pgp" ]; then
	if [ "${USE_PGP}" = "PGP" ]; then $PROGRAM --setup-pgp ; fi
	if [ "${USE_PGP}" = "GnuPG" ]; then $PROGRAM --setup-gnupg ; fi
	exit 0
fi

if [ "$1" = "--setup-pgp" ]; then
	echo "PGP not supported yet. sorry"
	exit 0
fi

if [ "$1" = "--setup-gnupg" ]; then
	GPGDIR=${LOGDIR}/gnupg
	GNUPG=`which gpg`
	if [ $? -eq 1 ]; then
		echo "Sorry: cannot find GnuPG executable"
		exit 0
	fi
	echo "Instalando Soporte para GNU Privacy Guard"
	mkdir -p ${GPGDIR}
	echo "Creando fichero de configuracion..."
	echo -n "Nombre del servidor de claves [ pgp.rediris.es ]: "
	read keyserver
	cat > ${GPGDIR}/options <<__EOF
no-greeting
#default-key 621CC013
#default-recipient some-user-id
#default-recipient-self
force-v3-sigs
escape-from-lines
#charset koi8-r
lock-once
#load-extension rndlinux
#load-extension rndunix
#load-extension rndegd
keyserver ${keyserver:=pgp.rediris.es}
honor-http-proxy
__EOF
	echo "Creando anillos de claves..."
cat > $TMPDIR/gnupg_data.$$ <<__EOF
%echo Generating a standard key
Key-Type: DSA
Key-Length: 1024
Subkey-Type: ELG-E
Subkey-Length: 1024
Name-Real: $ADMIN_FIRSTNAME $ADMIN_LASTNAME
Name-Comment: $DBTITLE
Name-Email: $ADMIN_EMAIL
Expire-Date: 0
Passphrase: $PGP_PASSWD
%pubring $LOGDIR/gnupg/pubring.gpg
%secring $LOGDIR/gnupg/secring.gpg
# Do a commit here, so that we can later print "done" :-)
%commit
%echo done
__EOF
	$GNUPG --batch --homedir ${GPGDIR} --gen-key $TMPDIR/gnupg_data.$$
	rm -f $TMPDIR/gnupg_data.$$
	# echo "Protecting secret key..."
	# echo $PGP_PASSWD | $GNUPG --batch --yes --passphrase-fd 0 --homedir ${GPGDIR} -o ${GPGDIR}/revoke_key --gen-revoke $ADMIN_EMAIL
	echo "Exportando clave publica..."
	echo $PGP_PASSWD | $GNUPG --homedir ${GPGDIR} --keyserver ${keyserver:=pgp.rediris.es} --passphrase-fd 0 --send-keys $ADMIN_EMAIL
	echo "Ajustando permisos..."
	chown -R nobody.root ${GPGDIR}
	echo "Proceso completado. "
	exit 0
fi

if [ "$1" = "--create-user" ]; then
	# Creamos el usuario administrador de la database. 
	# Puesto que la linea de comandos no permite meter la contrasea en
	# batch, usamos la database "template1" (standard del sistema) 
	# para operar como usuario postgres. Un poco chapuza, pero funciona
	com="CREATE USER $DBUSER WITH PASSWORD '$DBPASSWD' CREATEDB NOCREATEUSER;"
	su - postgres -c "/usr/bin/psql template1 -F : -h $DBHOST -p $DBPORT -t -A -q -c \"$com\""
	echo "Operador de la base de datos registrado"
	exit 0
fi

if [ "$1" = "--restart-db" ]; then
echo "Regenerando database..."
dropdb $DBNAME -U $DBUSER -h $DBHOST -p $DBPORT
createdb $DBNAME -U $DBUSER -h $DBHOST -p $DBPORT
createlang -U postgres -L /usr/lib/pgsql -h $DBHOST -p $DBPORT plpgsql $DBNAME
psql $DBNAME -U $DBUSER -h $DBHOST -p $DBPORT -f ${BASE}/lib/database.sql
echo "Hecho."
exit 0
fi

if [ "$1" = "--install-files" ]; then
# Generamos directorios
echo -n "Creando directorios..."
mkdir -p $INSTALLDIR $LOGDIR $BACKUPDIR
chown -R nobody.root $LOGDIR $BACKUPDIR
echo "Hecho."

# Copiamos ficheros
echo -n "Copiando ficheros...."
cp -rp $BASE/html/* $INSTALLDIR
cp $BASE/lib/database.sql $INSTALLDIR
cp $BASE/lib/* $BACKUPDIR 2> /dev/null
echo "Hecho."

# Personalizamos instalacin
echo -n "Personalizando ...."
# -1- crear script php con copia del /etc/freevote.conf
(
echo "<?";
# truco cutre para "php-izar" un script de asignaciones shell
sed 	-e 's/#.*//g' \
	-e '/^[ 	]*$/d' \
	-e '/DBPASSWD=/d' \
	-e '/DBUSER=/d' \
	-e '/PGP_PASSWD=/d' \
	-e 's/"//g' \
	-e 's/^[ 	]*/$/g' \
	-e 's/[ 	]*=[ 	]*/ = "/g' \
	-e 's/[ 	]*$/" ;/g' \
	-e 's/EVENT_\(.*\) = "\(.*\)"/EVENT_\1 = \2/g' \
	-e 's/SYSLOG = "\(.*\)"/SYSLOG = \1/g' /etc/freevote.conf
echo "?>"
) > $INSTALLDIR/config.php3
# -2- Generar datos para el httpd.conf
INST=`echo $INSTALLDIR | sed -e 's/\//\\\\\//g'`
sed -e "s/__INSTALLDIR__/${INST}/g" -e "s/__DBUSER__/${DBUSER}/g" -e "s/__DBPASSWD__/${DBPASSWD}/g" -e "s/__PGP_PASSWD__/${PGP_PASSWD}/g" < $BASE/lib/httpd.conf.freevote > ${APACHE_CONF}/httpd.conf.freevote
chmod 400 ${APACHE_CONF}/httpd.conf.freevote
grep -s -q freevote ${APACHE_CONF}/httpd.conf || echo "Include ${APACHE_CONF}/httpd.conf.freevote" >> ${APACHE_CONF}/httpd.conf
echo "Hecho."
exit 0

fi

if [ "$1" = "--register-sysop" ]; then
	echo "Registrando administrador de la base de datos...."
	mkdir -p $BACKUPDIR && chmod 700 $BACKUPDIR

	key=`${FREEVOTE_INST}/bin/freevote_crypt $ADMIN_EMAIL`
	sqlkey=`${FREEVOTE_INST}/bin/freevote_crypt $key $key`

	com="INSERT INTO usuarios \
		(groupid,passwd,nombre,apellidos,direccion,telefono,email) \
	VALUES ( 6,'$sqlkey','$ADMIN_FIRSTNAME','$ADMIN_LASTNAME','$ADMIN_ADDRESS','$ADMIN_PHONE','$ADMIN_EMAIL');"
	sqlquery $com 2> /dev/null
	if [ $? -ne 0 ]; then
	    echo "Error en INSERT: Posiblemente el usuario est ya registrado"
	    echo "Deber proceder a la instalacin manualmente"
	    exit 1
	fi
	com="SELECT userid FROM usuarios WHERE email='$ADMIN_EMAIL';"
	userid=`sqlquery $com`
	echo "Hecho. "
	echo "El userid del administrador es $userid"
	echo "La contrasea es $key"
	echo "La direccin de correo es $ADMIN_EMAIL"
	# guardamos datos en fichero del usuario
	cp /etc/freevote.conf $BACKUPDIR/freevote.conf
	echo "# Datos del administrador generados en la database" >> $BACKUPDIR/freevote.conf
	echo "ADMIN_USERID=$userid"	>> $BACKUPDIR/freevote.conf
	echo "ADMIN_PASSWD=$key"	>> $BACKUPDIR/freevote.conf
	chmod 400 $BACKUPDIR/freevote.conf
	echo "Una copia de los datos de instalacin y configuracin se ha guardado en $BACKUPDIR/freevote.conf"
	exit 0
fi

# si llega hasta aqu es que no han especificado parametros
echo "Uso: $0 opcion"
echo "Opciones disponibles:"
echo "	--do-all 		Proceso de instalacin completo"
echo "	--create-user	Define el propietario de la base de datos"
echo "	--restart-db	Reinicia la base de datos"
echo "	--install-pgp	Instala soporte para signado del correo"
echo "	--install-files 	Configura e instala los ficheros"
echo "	--register-sysop 	Registra administrador en la database"
