-- generador de secuencias de usuarios y votaciones y listas de votantes
CREATE SEQUENCE userid_seq;
CREATE SEQUENCE votacion_seq;
CREATE SEQUENCE lista_seq;
CREATE SEQUENCE comment_seq;

-- tabla de usuarios
CREATE TABLE usuarios (
	userid int4 DEFAULT nextval('userid_seq'),
	groupid int4 DEFAULT 1,
	passwd text NOT NULL DEFAULT '--NOLOGIN--',
	nombre text NOT NULL,
	apellidos text NOT NULL,
	direccion text NOT NULL,
	telefono text NOT NULL,
	email text UNIQUE NOT NULL,
	pgpsign text NOT NULL DEFAULT '',
	PRIMARY KEY ( userid )
);

-- Tabla con las listas definidas
CREATE TABLE listas (
	id_lista int4 DEFAULT nextval('lista_seq'),
	nombre	text UNIQUE NOT NULL,
	descripcion text NOT NULL,
	email text NOT NULL,
	PRIMARY KEY (id_lista)
);

-- Tabla con las diversas encuestas
CREATE TABLE consultas (
	voteid  int4 DEFAULT nextval('votacion_seq'),
	titulo text UNIQUE NOT NULL,
	userid int4 NOT NULL DEFAULT 0,
	preambulo text NOT NULL,
	pregunta text NOT NULL,
	-- fechas
	start_convocatoria date NOT NULL,
	end_convocatoria date NOT NULL,
	start_inscripcion date NOT NULL,
	end_inscripcion date NOT NULL,
	start_votacion date NOT NULL,
	end_votacion date NOT NULL,
	-- voting flags
	id_lista int4 NOT NULL DEFAULT 0,
	is_restricted bool NOT NULL DEFAULT 'false',
	is_secret bool NOT NULL DEFAULT 'true',
	has_forum bool NOT NULL DEFAULT 'true',
	-- result flags
	allow_percent bool NOT NULL DEFAULT 'true',
	allow_results bool NOT NULL DEFAULT 'false',
	allow_list bool NOT NULL DEFAULT 'false',
	allow_vote bool NOT NULL DEFAULT 'false', -- no usado
	-- claves
	PRIMARY KEY (voteid),
	FOREIGN KEY (userid) REFERENCES usuarios
		MATCH FULL 
		ON DELETE SET DEFAULT 
		ON UPDATE CASCADE,
	FOREIGN KEY (id_lista) REFERENCES listas
		MATCH FULL 
		ON DELETE SET DEFAULT 
		ON UPDATE CASCADE
);

-- respuesta asociadas a una votacion
CREATE TABLE respuestas (
	id_consulta int4 NOT NULL,
	short_answer text NOT NULL,
	long_answer text NOT NULL,
	result int4 DEFAULT 0,
	PRIMARY KEY (id_consulta,short_answer),
	FOREIGN KEY (id_consulta) REFERENCES consultas	
		MATCH FULL 
		ON DELETE CASCADE 
		ON UPDATE CASCADE
);

-- usuarios registrados para una votacion
CREATE TABLE votantes (
	id_consulta int4 NOT NULL,
	id_usuario int4 NOT NULL DEFAULT 0,
	vote_key text NOT NULL,
	ha_votado bool DEFAULT 'false',
	vote_data text DEFAULT '',
	PRIMARY KEY (id_consulta,id_usuario),
	FOREIGN KEY (id_consulta) REFERENCES consultas
		MATCH FULL 
		ON DELETE CASCADE 
		ON UPDATE CASCADE,
	FOREIGN KEY (id_usuario) REFERENCES usuarios
		MATCH FULL 
		ON DELETE CASCADE
		ON UPDATE CASCADE
);

-- listas de acceso para votaciones restringidas
CREATE TABLE user_acl (
	id_lista int4 NOT NULL DEFAULT 0,
	id_usuario int4 NOT NULL,
	fecha_alta date NOT NULL DEFAULT date(now()),
	UNIQUE (id_lista,id_usuario),
	FOREIGN KEY (id_usuario) REFERENCES usuarios
		MATCH FULL 
		ON DELETE CASCADE 
		ON UPDATE CASCADE,
	FOREIGN KEY (id_lista) REFERENCES listas
		MATCH FULL 
		ON DELETE CASCADE 
		ON UPDATE CASCADE
);

-- Listas de interventores 
-- el interventor no tiene por que estar registrado: se identifica por
-- el email

CREATE TABLE interventores (
	-- datos del interventor
        id_consulta int4 NOT NULL,
        id_usuario int4 NOT NULL,
	-- caracteristicas de la intervencion
	envia_lista bool NOT NULL DEFAULT 'true',
	envia_cambios bool NOT NULL DEFAULT 'true',
	envia_tarjetas bool NOT NULL DEFAULT 'true',
	envia_voto bool NOT NULL DEFAULT 'true',
	send_modo int4 NOT NULL DEFAULT 0,
        UNIQUE (id_consulta,id_usuario),
        FOREIGN KEY (id_usuario) REFERENCES usuarios
                MATCH FULL
                ON DELETE CASCADE
                ON UPDATE CASCADE,
        FOREIGN KEY (id_consulta) REFERENCES consultas
                MATCH FULL
                ON DELETE CASCADE
                ON UPDATE CASCADE
);

-- Tablas del foro de comentarios a las encuestas
CREATE TABLE foro (
	commentid int4 NOT NULL DEFAULT nextval('comment_seq'),
	userid int4 NOT NULL,
	voteid int4 NOT NULL,
	in_response_to int4,
	subject text NOT NULL DEFAULT 'Pon tu comentario',
	fecha timestamp NOT NULL DEFAULT now(),
	comment text,
	PRIMARY KEY (commentid),
	FOREIGN KEY (userid) REFERENCES usuarios
		MATCH FULL 
		ON DELETE SET DEFAULT
		ON UPDATE CASCADE,
	FOREIGN KEY (voteid) REFERENCES consultas
		MATCH FULL 
		ON DELETE CASCADE
		ON UPDATE CASCADE
);

-- ******************************************************************
-- listas temporales
-- ******************************************************************

-- tabla de usuarios pendientes de registro
CREATE TABLE usuarios_tmp (
	userid int4 DEFAULT nextval('userid_seq'),
	groupid int4 DEFAULT 1,
	passwd text NOT NULL DEFAULT '--NOLOGIN--',
	nombre text NOT NULL,
	apellidos text NOT NULL,
	direccion text NOT NULL,
	telefono text NOT NULL,
	email text UNIQUE NOT NULL,
	pgpsign text NOT NULL DEFAULT '',
	PRIMARY KEY ( userid )
);

-- tabla de votaciones pendientes de aprobacin
-- el usar id_consulta como contador nos garantiza unicidad entre ids de
-- tablas temporales y definitivas

CREATE TABLE consultas_tmp (
	voteid  int4 DEFAULT nextval('votacion_seq'),
	titulo text UNIQUE NOT NULL,
	userid int4 NOT NULL DEFAULT 0,
	preambulo text NOT NULL,
	pregunta text NOT NULL,
	-- fechas
	start_convocatoria date NOT NULL,
	end_convocatoria date NOT NULL,
	start_inscripcion date NOT NULL,
	end_inscripcion date NOT NULL,
	start_votacion date NOT NULL,
	end_votacion date NOT NULL,
	-- voting flags
	id_lista int4 NOT NULL DEFAULT 0,
	is_restricted bool NOT NULL DEFAULT 'false',
	is_secret bool NOT NULL DEFAULT 'true',
	has_forum bool NOT NULL DEFAULT 'true',
	-- result flags
	allow_percent bool NOT NULL DEFAULT 'true',
	allow_results bool NOT NULL DEFAULT 'false',
	allow_list bool NOT NULL DEFAULT 'false',
	allow_vote bool NOT NULL DEFAULT 'false', -- no usado
	-- claves
	PRIMARY KEY (voteid),
	FOREIGN KEY (userid) REFERENCES usuarios 
		MATCH FULL 
		ON DELETE SET DEFAULT 
		ON UPDATE CASCADE,
	FOREIGN KEY (id_lista) REFERENCES listas 
		MATCH FULL 
		ON DELETE SET DEFAULT 
		ON UPDATE CASCADE
);

-- respuesta asociadas a una votacion pendiente de registro

CREATE TABLE respuestas_tmp (
	id_consulta int4 NOT NULL,
	short_answer text NOT NULL,
	long_answer text NOT NULL,
	result int4 DEFAULT 0,
	PRIMARY KEY (id_consulta,short_answer),
	FOREIGN KEY (id_consulta) REFERENCES consultas_tmp	
		MATCH FULL 
		ON DELETE CASCADE 
		ON UPDATE CASCADE
);

-- *********************************************************
-- Funciones y triggers
-- *********************************************************

-- NOTA: En lugar de esto, utilizar el comando siguiente:
-- "postgres$ createlang -U postgres -L /usr/lib/pgsql plpgsql evote"
--
-- primero instalamos el manejador de PL/pgSQL
-- CREATE FUNCTION plpgsql_call_handler () RETURNS OPAQUE AS
--	'/usr/lib/pgsql/plpgsql.so' LANGUAGE 'C';
--
-- CREATE TRUSTED PROCEDURAL LANGUAGE 'plpgsql'
--	HANDLER plpgsql_call_handler LANCOMPILER 'PL/pgSQL';

-- Al dar de alta una consulta se crea un foro de discusion para esta
CREATE FUNCTION insert_comment() RETURNS OPAQUE AS ' 
	BEGIN
		INSERT INTO foro( userid, voteid)
		    VALUES(NEW.userid, NEW.voteid );
		RETURN NEW;
	END;
' LANGUAGE 'plpgsql' ;

CREATE TRIGGER do_insert_comment AFTER INSERT ON consultas 
	FOR EACH ROW EXECUTE PROCEDURE insert_comment();

-- Por defecto, al dar de alta a un usuario se le mete en la lista 
-- global, salvo que el usuario sea "nobody"
-- Del mismo modo, al dar de alta una lista, se le asigna a esta el
-- usuario "nobody", salvo que la lista sea la global

-- El motivo de estas excepciones es el de evitar bucles y errores en
-- los disparadores al crear las estructuras de datos iniciales.
-- Recuerda que el usuario nobody y la lista global son entes
-- predefinidos, y con identificadores prefijados al crear la database...

CREATE FUNCTION insert_acl() RETURNS OPAQUE AS ' 
	BEGIN
		IF NEW.userid = 0  THEN 
			RETURN NEW; 
		END IF;
		INSERT INTO user_acl VALUES(0,NEW.userid);
		RETURN NEW;
	END;
' LANGUAGE 'plpgsql' ;

CREATE TRIGGER do_insert_acl AFTER INSERT ON usuarios 
	FOR EACH ROW EXECUTE PROCEDURE insert_acl();

CREATE FUNCTION insert_into_list() RETURNS OPAQUE AS '
        BEGIN
		IF NEW.id_lista = 0  THEN 
			RETURN NEW; 
		END IF;
                INSERT INTO user_acl( id_lista,id_usuario)
                    VALUES( NEW.id_lista, 0 );
                RETURN NEW;
        END;
' LANGUAGE 'plpgsql' ;                                                          

CREATE TRIGGER do_insert_into_list AFTER INSERT ON listas
        FOR EACH ROW EXECUTE PROCEDURE insert_into_list();                            
-- funcion que indica si un usuario pertenece a una lista
CREATE FUNCTION pertenece(int4, int4) RETURNS bool AS '
DECLARE
	tmprow RECORD;
BEGIN
	SELECT * INTO tmprow FROM user_acl WHERE id_usuario=$1 AND id_lista=$2;
	IF NOT FOUND THEN RETURN ''f'';
	END IF;
	RETURN ''t'';
END;
' LANGUAGE 'plpgsql';

-- *********************************************************
-- Vistas y tablas accesibles para usuarios www y nobody
-- *********************************************************

-- (por definir.... )

-- ****************************************************************
-- Permisos de acceso
--
-- Nota: a partir de la version 1.0.2 no se accede como
-- usuario nobody, sino como usuario operador 
-- Del mismo modo los datos se toman desde variables de entorno
-- no desde ficheros de configuracion
-- ****************************************************************

-- Primero se borran todos los permisos
REVOKE ALL ON usuarios, consultas, respuestas, votantes, listas, user_acl, usuarios_tmp, consultas_tmp, respuestas_tmp, foro FROM nobody;

REVOKE ALL ON usuarios, consultas, respuestas, votantes, listas, user_acl, usuarios_tmp, consultas_tmp, respuestas_tmp, foro FROM www;

-- Permisos para el operdor...
GRANT ALL ON  consultas,consultas_tmp,respuestas,respuestas_tmp,listas,lista_seq,user_acl,userid_seq,usuarios,usuarios_tmp,votacion_seq,votantes,comment_seq,foro TO operador; 

-- Permisos para el usuario anonimo
-- GRANT SELECT,INSERT,DELETE ON consultas,usuarios,votantes,respuestas,usuarios_tmp,consultas_tmp,respuestas_tmp,listas,user_acl,foro TO nobody;

-- Para votar se permite el update en respuestas...
-- GRANT UPDATE ON respuestas to nobody;

-- Para actualizar datos de usuario se permite esto... posibles
-- problemas de seguridad. Aqu se debera usar una vista en lugar de ir
-- "a lo bestia" por la tabla
-- GRANT UPDATE ON usuarios to nobody;

-- *********************************************************
-- Inicializaciones
--
-- Truco: las secuencias empiezan con 1, luego nosotros ponemos "a capon"
-- el valor cero en el id. No puede fallar, pues esta en el script de
-- inicializacion
-- *********************************************************

-- Definicion de la lista universal ( todos los usuarios )
INSERT INTO listas VALUES (0,'Pblica','Todos los usuarios registrados','nolist@dot.com');

-- Usuario por defecto: 
-- Al borrar usuarios, las encuestas y grupos asociados, 
-- 	se reasignan a userid '0'
INSERT INTO usuarios VALUES(0,0,'--NOLOGIN--','Juan','Nadie','NoHome','0','nomail@dot.com');

